@@ -1,21 +1,28 @@
 package Catalyst::Plugin::Session::Store::FastMmap;
 
 use strict;
-use base
-  qw/Class::Data::Inheritable Class::Accessor::Fast Catalyst::Plugin::Session::Store/;
+use base qw/
+    Class::Data::Inheritable 
+    Class::Accessor::Fast 
+    Catalyst::Plugin::Session::Store/;
+
 use NEXT;
-use Cache::FastMmap;
+
+BEGIN {
+    require Cache::FastMmap            if $^O ne 'MSWin32';
+    require Cache::FastMmap::WithWin32 if $^O eq 'MSWin32';
+}
 use Path::Class     ();
 use File::Spec      ();
 use Catalyst::Utils ();
 
-our $VERSION = '0.01';
+our $VERSION = '0.03';
 
 __PACKAGE__->mk_classdata(qw/_session_fastmmap_storage/);
 
 =head1 NAME
 
-Catalyst::Plugin::Session::FastMmap - FastMmap sessions for Catalyst
+Catalyst::Plugin::Session::Store::FastMmap - FastMmap session storage backend.
 
 =head1 SYNOPSIS
 
@@ -33,7 +40,7 @@ Catalyst::Plugin::Session::FastMmap - FastMmap sessions for Catalyst
 
 C<Catalyst::Plugin::Session::Store::FastMmap> is a fast session storage plugin
 for Catalyst that uses an mmap'ed file to act as a shared memory interprocess
-cache. It is based on C<Cache::FastMmap>.
+cache. It is based on L<Cache::FastMmap>.
 
 =head2 METHODS
 
@@ -50,6 +57,11 @@ cache. It is based on C<Cache::FastMmap>.
 These are implementations of the required methods for a store. See
 L<Catalyst::Plugin::Session::Store>.
 
+=item get_and_set_session_data
+
+This is the optional method for atomic write semantics. See
+L<Catalyst::Plugin::Session::AtomicWrite>.
+
 =cut
 
 sub get_session_data {
@@ -67,9 +79,15 @@ sub delete_session_data {
     $c->_session_fastmmap_storage->remove($sid);
 }
 
-sub delete_expired_sessions {
-    my $c = shift;
-    $c->_session_fastmmap_storage->empty(1);    # 1 means only expired
+sub delete_expired_sessions { } # unsupported
+
+sub get_and_set_session_data {
+    my ( $c, $sid, $sub ) = @_;
+    $c->_session_fastmmap_storage->get_and_set( $sid, sub {
+        my ( $key, $data ) = @_;
+        my $new = $sub->( $key, $data ); 
+        return $new;
+    });
 }
 
 =item setup_session
@@ -98,8 +116,8 @@ sub setup_session {
 
     $c->_session_fastmmap_storage(
         Cache::FastMmap->new(
+            raw_values  => 0,
             share_file  => $cfg->{storage},
-            expire_time => $cfg->{expires},
             (
                 map { $_ => $cfg->{$_} }
                   grep { exists $cfg->{$_} } qw/init_file cache_size/
@@ -117,6 +135,9 @@ expired prematurely, due to the LRU caching policy employed by
 L<Cache::FastMmap>. To get around this you can increase the C<cache_size>
 parameter, or switch session storage backends.
 
+This is particularly inappropriate for use as a backend for e.g.
+L<Catalyst::Plugin::Session::PerUser>, for example.
+
 L<Cache::FastMmap> defaults to around 5mb (89 * 64k).
 
 =head1 CONFIGURATION
